@@ -5,6 +5,8 @@
 #include "KasumiWord.hxx"
 #include "KasumiDic.hxx"
 #include "KasumiString.hxx"
+#include "KasumiAddWindow.hxx"
+#include "KasumiConfiguration.hxx"
 #include <gdk/gdkkeysyms.h>
 #include "intl.h"
 
@@ -14,7 +16,10 @@
 
 using namespace std;
 
-KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary){
+KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary,
+                                   KasumiConfiguration *conf){
+  this->conf = conf;
+  
   dictionary = aDictionary;
   modificationFlag = false;
   
@@ -22,8 +27,6 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary){
   gtk_window_set_title(GTK_WINDOW(window), _("Kasumi"));
   g_signal_connect(G_OBJECT(window), "delete_event",
                    G_CALLBACK(_call_back_delete_event), this);
-  g_signal_connect(G_OBJECT(window), "destroy",
-                   G_CALLBACK(_call_back_destroy), this);
 
   // creating top vbox
   GtkWidget *vbox = gtk_vbox_new(FALSE,8);
@@ -90,7 +93,7 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary){
   GtkWidget *label = gtk_label_new(_("Spelling"));
   GtkWidget *alignment = gtk_alignment_new(0,0.5,0,0);
   gtk_container_add(GTK_CONTAINER(alignment),GTK_WIDGET(label));
-  gtk_box_pack_start(GTK_BOX(entry_vbox),GTK_WIDGET(alignment),FALSE,FALSE,0);
+  gtk_box_pack_start(GTK_BOX(entry_vbox),GTK_WIDGET(alignment),FALSE,FALSE,2);
 
   SpellingEntry = gtk_entry_new();
   HandlerIDOfSpellingEntry =
@@ -102,25 +105,29 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary){
                      GTK_WIDGET(SpellingEntry),
                      FALSE,
                      FALSE,
-                     0);
+                     2);
 
   // creating text entries for "Sound"
   label = gtk_label_new(_("Sound"));
   alignment = gtk_alignment_new(0,0.5,0,0);
   gtk_container_add(GTK_CONTAINER(alignment),GTK_WIDGET(label));
-  gtk_box_pack_start(GTK_BOX(entry_vbox),GTK_WIDGET(alignment),FALSE,FALSE,0);
+  gtk_box_pack_start(GTK_BOX(entry_vbox),GTK_WIDGET(alignment),FALSE,FALSE,2);
 
   SoundEntry = gtk_entry_new();
   HandlerIDOfSoundEntry = g_signal_connect(G_OBJECT(SoundEntry), "changed",
                G_CALLBACK(_call_back_changed_sound_entry), this);
-  gtk_box_pack_start(GTK_BOX(entry_vbox),GTK_WIDGET(SoundEntry),FALSE,FALSE,0);
+  gtk_box_pack_start(GTK_BOX(entry_vbox),GTK_WIDGET(SoundEntry),FALSE,FALSE,2);
+  previousSoundEntryText = string(gtk_entry_get_text(GTK_ENTRY(SoundEntry)));
 
   // creating spin button for "Frequency"
   label = gtk_label_new(_("Frequency"));
   alignment = gtk_alignment_new(0,0.5,0,0);
   gtk_container_add(GTK_CONTAINER(alignment),GTK_WIDGET(label));
-  gtk_box_pack_start(GTK_BOX(entry_vbox),GTK_WIDGET(alignment),FALSE,FALSE,0);
+  gtk_box_pack_start(GTK_BOX(entry_vbox),GTK_WIDGET(alignment),FALSE,FALSE,2);
 
+  const int FREQ_DEFAULT = conf->getPropertyValueByInt("DefaultFrequency");
+  const int FREQ_LBOUND = conf->getPropertyValueByInt("MinFrequency");
+  const int FREQ_UBOUND = conf->getPropertyValueByInt("MaxFrequency");
   GtkObject *adjustment = gtk_adjustment_new(FREQ_DEFAULT,
                                              FREQ_LBOUND,
                                              FREQ_UBOUND,
@@ -137,13 +144,13 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary){
                      GTK_WIDGET(FrequencySpin),
                      FALSE,
                      FALSE,
-                     0);
+                     2);
 
   // creating combo box for "Word Class"
   label = gtk_label_new(_("Word Class"));
   alignment = gtk_alignment_new(0,0.5,0,0);
   gtk_container_add(GTK_CONTAINER(alignment),GTK_WIDGET(label));
-  gtk_box_pack_start(GTK_BOX(entry_vbox),GTK_WIDGET(alignment),FALSE,FALSE,0);
+  gtk_box_pack_start(GTK_BOX(entry_vbox),GTK_WIDGET(alignment),FALSE,FALSE,2);
   
   WordClassCombo = gtk_combo_box_new_text();
   gtk_combo_box_append_text(GTK_COMBO_BOX(WordClassCombo), _("Noun"));  
@@ -151,12 +158,13 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary){
   gtk_combo_box_append_text(GTK_COMBO_BOX(WordClassCombo), _("Person's name"));
   gtk_combo_box_append_text(GTK_COMBO_BOX(WordClassCombo), _("Place-name"));
   gtk_combo_box_append_text(GTK_COMBO_BOX(WordClassCombo), _("Adjective"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(WordClassCombo), _("Verb"));
   gtk_combo_box_set_active(GTK_COMBO_BOX(WordClassCombo), 0);
   HandlerIDOfWordClassCombo =
     g_signal_connect(G_OBJECT(WordClassCombo), "changed",
                      G_CALLBACK(_call_back_changed_word_class_combo), this);
   gtk_box_pack_start(GTK_BOX(entry_vbox),
-                     GTK_WIDGET(WordClassCombo),FALSE,FALSE,0);
+                     GTK_WIDGET(WordClassCombo),FALSE,FALSE,2);
 
   // creating noun option pane
   NounOptionPane = gtk_vbox_new(FALSE,0);
@@ -249,33 +257,76 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary){
     g_signal_connect(G_OBJECT(AdvOptionGokanCheck),"toggled",
                      G_CALLBACK(_call_back_toggled_check),this);
 
+  // creating verb option pane
+  VerbOptionPane = gtk_vbox_new(FALSE,0);
+  gtk_box_pack_start(GTK_BOX(entry_vbox),GTK_WIDGET(VerbOptionPane),
+                     FALSE,FALSE,0);
+
+  label = gtk_label_new(_("VerbType"));
+  alignment = gtk_alignment_new(0,0.5,0,0);
+  gtk_container_add(GTK_CONTAINER(alignment),GTK_WIDGET(label));
+  gtk_box_pack_start(GTK_BOX(VerbOptionPane),GTK_WIDGET(alignment),
+                             FALSE,FALSE,2);
+
+  VerbTypeCombo = gtk_combo_box_new_text();
+  gtk_combo_box_append_text(GTK_COMBO_BOX(VerbTypeCombo), _("Ba line, 5 columns"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(VerbTypeCombo), _("Ga line, 5 columns"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(VerbTypeCombo), _("Ka line, 5 columns"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(VerbTypeCombo), _("Ma line, 5 columns"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(VerbTypeCombo), _("Na line, 5 columns"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(VerbTypeCombo), _("Ra line, 5 columns"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(VerbTypeCombo), _("Sa line, 5 columns"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(VerbTypeCombo), _("Ta line, 5 columns"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(VerbTypeCombo), _("Wa line, 5 columns"));
+  gtk_combo_box_set_active(GTK_COMBO_BOX(VerbTypeCombo), 0);
+  HandlerIDOfVerbTypeCombo =
+    g_signal_connect(G_OBJECT(VerbTypeCombo), "changed",
+                     G_CALLBACK(_call_back_changed_verb_type_combo), this);
+  gtk_box_pack_start(GTK_BOX(VerbOptionPane),
+                     GTK_WIDGET(VerbTypeCombo),
+                     FALSE,FALSE,0);
+
+  VerbOptionRentaiCheck =
+    gtk_check_button_new_with_label(_("Can be Meishi when Renyou type"));
+  gtk_box_pack_start(GTK_BOX(VerbOptionPane),
+                     GTK_WIDGET(VerbOptionRentaiCheck),
+                     FALSE,FALSE,0);
+  HandlerIDOfVerbOptionRentaiCheck =
+    g_signal_connect(G_OBJECT(VerbOptionRentaiCheck),"toggled",
+                     G_CALLBACK(_call_back_toggled_check),this);
+
+
   // creating box for search function
   GtkWidget *search_hbox = gtk_hbox_new(FALSE,8);
   gtk_box_pack_start(GTK_BOX(vbox),GTK_WIDGET(search_hbox),FALSE,FALSE,0);
 
   // creating Entry and Button for search function
   SearchEntry = gtk_entry_new();
+  g_signal_connect(G_OBJECT(SearchEntry),
+                   "changed",
+                   G_CALLBACK(_call_back_changed_search_entry),
+                   this);
+  g_signal_connect(G_OBJECT(SearchEntry),
+                   "activate",
+                   G_CALLBACK(_call_back_activate_search_entry),
+                   this); // called when Entry key is pressed
   gtk_box_pack_start(GTK_BOX(search_hbox),
                      GTK_WIDGET(SearchEntry),
                      FALSE,
                      FALSE,
                      0);
 
-  GtkWidget *button = gtk_button_new();
-  gtk_button_set_label(GTK_BUTTON(button),_("Find Next By Sound"));
-  gtk_box_pack_start(GTK_BOX(search_hbox),GTK_WIDGET(button),FALSE,FALSE,0);
-  g_signal_connect(G_OBJECT(button),"clicked",
-                   G_CALLBACK(_call_back_find_next_by_sound),this);
+  SearchBySpellingRadio = gtk_radio_button_new_with_label(NULL,
+                                                          _("Find By Spelling"));
+  gtk_box_pack_start(GTK_BOX(search_hbox),SearchBySpellingRadio,
+                     TRUE,TRUE,0);
+  
+  SearchBySoundRadio =
+    gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(SearchBySpellingRadio),
+                                                _("Find By Sound"));
+  gtk_box_pack_start(GTK_BOX(search_hbox),SearchBySoundRadio,
+                     TRUE,TRUE,0);
 
-  button = gtk_button_new();
-  gtk_button_set_label(GTK_BUTTON(button),_("Find Next By Spelling"));
-  gtk_box_pack_start(GTK_BOX(search_hbox),GTK_WIDGET(button),FALSE,FALSE,0);
-  g_signal_connect(G_OBJECT(button),"clicked",
-                   G_CALLBACK(_call_back_find_next_by_spelling),this);
-
-  PrefixSearchCheck = gtk_check_button_new_with_label(_("Prefix Search"));
-  gtk_box_pack_start(GTK_BOX(search_hbox),GTK_WIDGET(PrefixSearchCheck),
-                     FALSE,FALSE,0);
 
   /* creating box for buttons */
   GtkWidget *hbutton_box = gtk_hbutton_box_new();
@@ -284,15 +335,18 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary){
 
 
   // creating buttons and shortcut key configuration
-  button = gtk_button_new();
+  GtkWidget *button = gtk_button_new();
   gtk_button_set_label(GTK_BUTTON(button),_("Quit"));
   gtk_box_pack_start(GTK_BOX(hbutton_box),GTK_WIDGET(button),FALSE,FALSE,0);
   g_signal_connect(G_OBJECT(button),"clicked",
                    G_CALLBACK(_call_back_quit),this);
   GtkAccelGroup *accel = gtk_accel_group_new();
   gtk_window_add_accel_group(GTK_WINDOW(window), accel);
+  string key = conf->getPropertyValue("QuitShortcutKey");
   gtk_widget_add_accelerator(button, "clicked", accel,
-                             GDK_Q, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
+                             getAccelKey(key),
+                             getModifierType(key),
+                             GTK_ACCEL_VISIBLE);
 
   button = gtk_button_new();
   gtk_button_set_label(GTK_BUTTON(button),_("Store"));
@@ -301,18 +355,24 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary){
                    G_CALLBACK(_call_back_store),this);
   accel = gtk_accel_group_new();
   gtk_window_add_accel_group(GTK_WINDOW(window), accel);
+  key = conf->getPropertyValue("StoreShortcutKey");
   gtk_widget_add_accelerator(button, "clicked", accel,
-                             GDK_S, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
+                             getAccelKey(key),
+                             getModifierType(key),
+                             GTK_ACCEL_VISIBLE);
 
   button = gtk_button_new();
-  gtk_button_set_label(GTK_BUTTON(button),_("Add"));
+  gtk_button_set_label(GTK_BUTTON(button),_("New Word"));
   gtk_box_pack_start(GTK_BOX(hbutton_box),GTK_WIDGET(button),FALSE,FALSE,0);
   g_signal_connect(G_OBJECT(button),"clicked",
                    G_CALLBACK(_call_back_add),this);
   accel = gtk_accel_group_new();
   gtk_window_add_accel_group(GTK_WINDOW(window), accel);
+  key = conf->getPropertyValue("NewWordShortcutKey");  
   gtk_widget_add_accelerator(button, "clicked", accel,
-                             GDK_A, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
+                             getAccelKey(key),
+                             getModifierType(key),
+                             GTK_ACCEL_VISIBLE);
   
   button = gtk_button_new();
   gtk_button_set_label(GTK_BUTTON(button),_("Remove"));
@@ -321,11 +381,35 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary){
                    G_CALLBACK(_call_back_remove),this);
   accel = gtk_accel_group_new();
   gtk_window_add_accel_group(GTK_WINDOW(window), accel);
+  key = conf->getPropertyValue("RemoveShortcutKey");
   gtk_widget_add_accelerator(button, "clicked", accel,
-                             GDK_D, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
-  
+                             getAccelKey(key),
+                             getModifierType(key),
+                             GTK_ACCEL_VISIBLE);
+
+  button = gtk_button_new();
+  gtk_button_set_label(GTK_BUTTON(button),_("Adding Mode"));
+  gtk_box_pack_start(GTK_BOX(hbutton_box),GTK_WIDGET(button),FALSE,FALSE,0);
+  g_signal_connect(G_OBJECT(button),"clicked",
+                   G_CALLBACK(_call_back_adding_mode),this);
+  accel = gtk_accel_group_new();
+  gtk_window_add_accel_group(GTK_WINDOW(window), accel);
+  key = conf->getPropertyValue("AddingModeShortcutKey");
+  gtk_widget_add_accelerator(button, "clicked", accel,
+                             getAccelKey(key),
+                             getModifierType(key),
+                             GTK_ACCEL_VISIBLE);
+
   gtk_widget_show_all(window);
   gtk_widget_hide(AdvOptionPane);
+  gtk_widget_hide(VerbOptionPane);
+
+  // set default window position
+  int x = conf->getPropertyValueByInt("DefaultWindowPosX");
+  int y = conf->getPropertyValueByInt("DefaultWindowPosY");
+  if(x >= 0 && y >= 0){
+    gtk_window_move(GTK_WINDOW(window),x,y);
+  }
 
   refresh();
 
@@ -333,6 +417,8 @@ KasumiMainWindow::KasumiMainWindow(KasumiDic *aDictionary){
 }
 
 void KasumiMainWindow::refresh(){
+  GtkTreeModel *model = GTK_TREE_MODEL(WordList);
+  GtkTreeIter iter;
   int i = 0;
 
   gtk_list_store_clear(WordList);
@@ -359,13 +445,37 @@ void KasumiMainWindow::refresh(){
       exit(1);
     }
   }
+
+  if(!gtk_tree_model_get_iter_first(model, &iter)){
+    // If no words, disable text entries and other visible widgets
+    gtk_widget_set_sensitive(SpellingEntry,false);
+    gtk_widget_set_sensitive(SoundEntry,false);
+    gtk_widget_set_sensitive(FrequencySpin,false);
+    gtk_widget_set_sensitive(WordClassCombo,false);
+    gtk_widget_set_sensitive(NounOptionSaConnectionCheck,false);
+    gtk_widget_set_sensitive(NounOptionNaConnectionCheck,false);
+    gtk_widget_set_sensitive(NounOptionSuruConnectionCheck,false);
+    gtk_widget_set_sensitive(NounOptionGokanCheck,false);
+    gtk_widget_set_sensitive(NounOptionKakujoshiConnectionCheck,false);
+
+    return;
+  }
+
+  // select first word
+  gtk_tree_selection_select_iter(WordListSelection,&iter);  
+
+}
+
+KasumiMainWindow::~KasumiMainWindow(){
+  dictionary->removeEventListener(this);
+  destroy();
 }
 
 void KasumiMainWindow::destroy(){
-  gtk_main_quit();
+  gtk_widget_destroy(window);
 }
 
-gboolean KasumiMainWindow::delete_event(GdkEvent *event){
+void KasumiMainWindow::quit(){
   if(modificationFlag){
     GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW(window),
                                      GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -386,13 +496,8 @@ gboolean KasumiMainWindow::delete_event(GdkEvent *event){
     }
     gtk_widget_destroy (dialog);
   }
-  return FALSE;
-}
-
-void KasumiMainWindow::ClickedQuitButton(){
-  if(!delete_event(NULL)){
-    destroy();
-  }
+  delete this;
+  gtk_main_quit();
 }
 
 void KasumiMainWindow::ClickedStoreButton(){
@@ -406,15 +511,7 @@ void KasumiMainWindow::ClickedStoreButton(){
 }
 
 void KasumiMainWindow::ClickedAddButton(){
-  KasumiWord *word = new KasumiWord();
-  word->setSoundByUTF8(string(gtk_entry_get_text(
-    GTK_ENTRY(SoundEntry))));
-  word->setSpellingByUTF8(string(gtk_entry_get_text(
-    GTK_ENTRY(SpellingEntry))));
-  word->setFrequency(gtk_spin_button_get_value_as_int(
-    GTK_SPIN_BUTTON(FrequencySpin)));
-  word->setWordClass(getActiveWordClass());
-
+  KasumiWord *word = new KasumiWord(conf);
   dictionary->appendWord(word);
 }
 
@@ -441,37 +538,46 @@ void KasumiMainWindow::ChangedListCursor(){
     g_signal_handler_block(SoundEntry, HandlerIDOfSoundEntry);
     g_signal_handler_block(SpellingEntry, HandlerIDOfSpellingEntry);
     g_signal_handler_block(FrequencySpin, HandlerIDOfFrequencySpin);
-    g_signal_handler_block(WordClassCombo, HandlerIDOfWordClassCombo);  
+    g_signal_handler_block(WordClassCombo, HandlerIDOfWordClassCombo);
+    g_signal_handler_block(VerbTypeCombo, HandlerIDOfVerbTypeCombo);
                            
     gtk_entry_set_text(GTK_ENTRY(SpellingEntry),
                        word->getSpellingByUTF8().c_str());
     gtk_entry_set_text(GTK_ENTRY(SoundEntry),
                        word->getSoundByUTF8().c_str());
+    previousSoundEntryText = string(gtk_entry_get_text(GTK_ENTRY(SoundEntry)));
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(FrequencySpin),
                               word->getFrequency());
     setActiveWordClass(word->getWordClass());
+    setActiveVerbType(word->getVerbType());
 
     g_signal_handler_unblock(SoundEntry, HandlerIDOfSoundEntry);
     g_signal_handler_unblock(SpellingEntry, HandlerIDOfSpellingEntry);
     g_signal_handler_unblock(FrequencySpin, HandlerIDOfFrequencySpin);
     g_signal_handler_unblock(WordClassCombo, HandlerIDOfWordClassCombo);
+    g_signal_handler_unblock(VerbTypeCombo, HandlerIDOfVerbTypeCombo);
 
     synchronizeOptionCheckButton(word);
   }else{
     g_signal_handler_block(SoundEntry, HandlerIDOfSoundEntry);
     g_signal_handler_block(SpellingEntry, HandlerIDOfSpellingEntry);
     g_signal_handler_block(FrequencySpin, HandlerIDOfFrequencySpin);
-    g_signal_handler_block(WordClassCombo, HandlerIDOfWordClassCombo);  
+    g_signal_handler_block(WordClassCombo, HandlerIDOfWordClassCombo);
+    g_signal_handler_block(VerbTypeCombo, HandlerIDOfVerbTypeCombo);
     
     gtk_entry_set_text(GTK_ENTRY(SpellingEntry),"");
     gtk_entry_set_text(GTK_ENTRY(SoundEntry),"");
+    previousSoundEntryText = string(gtk_entry_get_text(GTK_ENTRY(SoundEntry)));
+    const int FREQ_DEFAULT = conf->getPropertyValueByInt("DefaultFrequency");
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(FrequencySpin),FREQ_DEFAULT);
     setActiveWordClass(NOUN);
+    setActiveVerbType(B5);
     
     g_signal_handler_unblock(SoundEntry, HandlerIDOfSoundEntry);
     g_signal_handler_unblock(SpellingEntry, HandlerIDOfSpellingEntry);
     g_signal_handler_unblock(FrequencySpin, HandlerIDOfFrequencySpin);
     g_signal_handler_unblock(WordClassCombo, HandlerIDOfWordClassCombo);
+    g_signal_handler_unblock(VerbTypeCombo, HandlerIDOfVerbTypeCombo);
     
     synchronizeOptionCheckButton(NULL);
   }
@@ -487,8 +593,30 @@ void KasumiMainWindow::ChangedSoundEntry(){
   if(gtk_tree_selection_get_selected(WordListSelection, &model, &iter)){
     gtk_tree_model_get(model, &iter, COL_ID, &id, -1);
     KasumiWord *word = dictionary->getWordWithID(id);
-    word->setSoundByUTF8(string(gtk_entry_get_text(GTK_ENTRY(SoundEntry))));
-    modifiedWord(id);
+    try{
+      word->setSoundByUTF8(string(gtk_entry_get_text(GTK_ENTRY(SoundEntry))));
+      dictionary->modifyWord(id);
+      previousSoundEntryText = string(gtk_entry_get_text(GTK_ENTRY(SoundEntry)));
+    }catch(KasumiInvalidCharacterForSoundException e){
+      string message;
+      message = string(_("Sound must consist of only Hiragana characters. You have entered invalid character: "));
+      message += e.getInvalidCharacter();
+      
+      GtkWidget *dialog = gtk_message_dialog_new
+        (GTK_WINDOW(window),
+         GTK_DIALOG_DESTROY_WITH_PARENT,
+         GTK_MESSAGE_ERROR,
+         GTK_BUTTONS_OK,
+         message.c_str());
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+
+      // take back invalid entry
+      g_signal_handler_block(SoundEntry, HandlerIDOfSoundEntry);
+      gtk_entry_set_text(GTK_ENTRY(SoundEntry),
+                         previousSoundEntryText.c_str());
+      g_signal_handler_unblock(SoundEntry, HandlerIDOfSoundEntry);
+    }
   }
 }
 
@@ -502,7 +630,8 @@ void KasumiMainWindow::ChangedSpellingEntry(){
     KasumiWord *word = dictionary->getWordWithID(id);
     word->setSpellingByUTF8(string(gtk_entry_get_text(
       GTK_ENTRY(SpellingEntry))));
-    modifiedWord(id);
+    //    modifiedWord(id);
+    dictionary->modifyWord(id);
   }
 }
 
@@ -516,7 +645,8 @@ void KasumiMainWindow::ChangedFrequencySpin(){
     KasumiWord *word = dictionary->getWordWithID(id);
     word->setFrequency(gtk_spin_button_get_value_as_int(
       GTK_SPIN_BUTTON(FrequencySpin)));
-    modifiedWord(id);
+    //modifiedWord(id);
+    dictionary->modifyWord(id);
   }
 }
 
@@ -530,12 +660,27 @@ void KasumiMainWindow::ChangedWordClassCombo(){
     KasumiWord *word = dictionary->getWordWithID(id);
     word->setWordClass(getActiveWordClass());
     synchronizeOptionCheckButton(word);
-    modifiedWord(id);
+    //    modifiedWord(id);
+    dictionary->modifyWord(id);
   }
 
   flipOptionPane();  
 }
 
+void KasumiMainWindow::ChangedVerbTypeCombo(){
+ GtkTreeModel *model = GTK_TREE_MODEL(WordList);
+  GtkTreeIter iter;
+  int id;
+  
+  if(gtk_tree_selection_get_selected(WordListSelection, &model, &iter)){
+    gtk_tree_model_get(model, &iter, COL_ID, &id, -1);
+    KasumiWord *word = dictionary->getWordWithID(id);
+    word->setVerbType(getActiveVerbType());
+    //    modifiedWord(id);
+    dictionary->modifyWord(id);
+  }
+}
+
 void KasumiMainWindow::ChangedOption(GtkWidget *widget){
   GtkTreeModel *model = GTK_TREE_MODEL(WordList);
   GtkTreeIter iter;
@@ -561,6 +706,8 @@ void KasumiMainWindow::ChangedOption(GtkWidget *widget){
     OptionName = string(EUCJP_SURUSETSUZOKU);    
   }else if(widget == AdvOptionGokanCheck){
     OptionName = string(EUCJP_GOKANNNOMIDEBUNNSETSU);
+  }else if(widget == VerbOptionRentaiCheck){
+    OptionName = string(EUCJP_RENNYOUKEINOMEISHIKA);
   }
 
   if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
@@ -573,31 +720,32 @@ void KasumiMainWindow::ChangedOption(GtkWidget *widget){
     gtk_tree_model_get(model, &iter, COL_ID, &id, -1);
     KasumiWord *word = dictionary->getWordWithID(id);
     word->setOption(OptionName, val);
-    modifiedWord(id);
+    //    modifiedWord(id);
+    dictionary->modifyWord(id);
   }
 }
 
-void KasumiMainWindow::FindNextBySound(GtkWidget *widget){
-  FindNext(SOUND);
+void KasumiMainWindow::SwitchToAddingMode(){
+  new KasumiAddWindow(dictionary,conf);
+  delete this;
 }
 
-void KasumiMainWindow::FindNextBySpelling(GtkWidget *widget){
-  FindNext(SPELLING);
-}
-
-void KasumiMainWindow::FindNext(SearchBy by){
+void KasumiMainWindow::FindNext(bool fromCurrent){
   GtkTreeModel *model = GTK_TREE_MODEL(WordList);
   GtkTreeIter iter;
   KasumiWord *word;
   bool fromFirst = false;
-  bool prefixSearch =
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(PrefixSearchCheck));
   GtkTreeIter StartIter;
   int id;
   string searchString = string(gtk_entry_get_text(GTK_ENTRY(SearchEntry)));
   string comparedString;
   GtkWidget *dialog;
 
+  SearchBy by = SPELLING;
+  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SearchBySoundRadio))){
+    by = SOUND;
+  }
+
   if(!gtk_tree_selection_get_selected(WordListSelection, &model, &iter)){
     if(!gtk_tree_model_get_iter_first(model, &iter)){
       // If no words, do nothing.
@@ -606,10 +754,14 @@ void KasumiMainWindow::FindNext(SearchBy by){
     fromFirst = true;
   }
   StartIter = iter;
-  
-  // Search from next word if a certain word is selected.
-  // If the selected is the last word, seek from the first word.
-  if(!fromFirst && !gtk_tree_model_iter_next(model,&iter)){
+
+  if(fromCurrent){
+    // Search from the selected world
+    // nothing to do ... discourage against calling gtk_tree_model_iter_next
+  }else if(!fromFirst && !gtk_tree_model_iter_next(model,&iter)){
+    // Search from next word if a certain word is selected.
+    // If the selected is the last word, seek from the first word.
+    
     // If no words, do nothing;    
     if(!gtk_tree_model_get_iter_first(model, &iter))
       return;
@@ -628,8 +780,7 @@ void KasumiMainWindow::FindNext(SearchBy by){
       comparedString = word->getSoundByUTF8();
     }
 
-    if((comparedString == searchString) ||
-       (prefixSearch && comparedString.find(searchString,0) == 0)){
+    if(comparedString.find(searchString,0) == 0){
       // if found, select that word and don't search any more
       gtk_tree_selection_select_iter(WordListSelection,&iter);
       return;
@@ -662,8 +813,7 @@ void KasumiMainWindow::FindNext(SearchBy by){
           comparedString = word->getSoundByUTF8();
         }
 
-        if((comparedString == searchString) ||
-           (prefixSearch && comparedString.find(searchString,0) == 0)){
+        if(comparedString.find(searchString,0) == 0){
           gtk_tree_selection_select_iter(WordListSelection,&iter);
           return;
         }
@@ -673,7 +823,7 @@ void KasumiMainWindow::FindNext(SearchBy by){
               StartIter.user_data3 != iter.user_data3));
     }else{
       gtk_widget_destroy(dialog);
-      gtk_tree_selection_unselect_all(WordListSelection);      
+      //      gtk_tree_selection_unselect_all(WordListSelection);      
       return;
     }
   }
@@ -685,7 +835,7 @@ void KasumiMainWindow::FindNext(SearchBy by){
                                   _("Cannot find a specific word."));
   gtk_dialog_run(GTK_DIALOG (dialog));
   gtk_widget_destroy(dialog);
-  gtk_tree_selection_unselect_all(WordListSelection);
+  //  gtk_tree_selection_unselect_all(WordListSelection);
 }
 
 void KasumiMainWindow::removedWord(int id){
@@ -698,6 +848,17 @@ void KasumiMainWindow::appendedWord(int id){
   KasumiWord *word = dictionary->getWordWithID(id);
   
   gtk_list_store_append(WordList,&iter);
+
+  // activate disabled widget
+  gtk_widget_set_sensitive(SpellingEntry,true);
+  gtk_widget_set_sensitive(SoundEntry,true);
+  gtk_widget_set_sensitive(FrequencySpin,true);
+  gtk_widget_set_sensitive(WordClassCombo,true);
+  gtk_widget_set_sensitive(NounOptionSaConnectionCheck,true);
+  gtk_widget_set_sensitive(NounOptionNaConnectionCheck,true);
+  gtk_widget_set_sensitive(NounOptionSuruConnectionCheck,true);
+  gtk_widget_set_sensitive(NounOptionGokanCheck,true);
+  gtk_widget_set_sensitive(NounOptionKakujoshiConnectionCheck,true);
   
   gtk_list_store_set(WordList, &iter,
                      COL_ID, id,
@@ -747,6 +908,66 @@ GtkTreeIter *KasumiMainWindow::findCorrespondingIter(int id){
   return NULL;
 }
 
+void KasumiMainWindow::setActiveVerbType(VerbType type){
+  switch(type){
+  case B5:
+    gtk_combo_box_set_active(GTK_COMBO_BOX(VerbTypeCombo),0);
+    break;
+  case G5:
+    gtk_combo_box_set_active(GTK_COMBO_BOX(VerbTypeCombo),1);
+    break;
+  case K5:
+    gtk_combo_box_set_active(GTK_COMBO_BOX(VerbTypeCombo),2);
+    break;
+  case M5:
+    gtk_combo_box_set_active(GTK_COMBO_BOX(VerbTypeCombo),3);
+    break;
+  case N5:
+    gtk_combo_box_set_active(GTK_COMBO_BOX(VerbTypeCombo),4);
+    break;
+  case R5:
+    gtk_combo_box_set_active(GTK_COMBO_BOX(VerbTypeCombo),5);
+    break;
+  case S5:
+    gtk_combo_box_set_active(GTK_COMBO_BOX(VerbTypeCombo),6);
+    break;
+  case T5:
+    gtk_combo_box_set_active(GTK_COMBO_BOX(VerbTypeCombo),7);
+    break;
+  case W5:
+    gtk_combo_box_set_active(GTK_COMBO_BOX(VerbTypeCombo),8);
+    break;
+  default:
+    gtk_combo_box_set_active(GTK_COMBO_BOX(VerbTypeCombo),0);
+    break;
+  }
+}
+
+VerbType KasumiMainWindow::getActiveVerbType(){
+  switch(gtk_combo_box_get_active(GTK_COMBO_BOX(VerbTypeCombo))){
+  case 0:
+    return B5;
+  case 1:
+    return G5;
+  case 2:
+    return K5;
+  case 3:
+    return M5;
+  case 4:
+    return N5;
+  case 5:
+    return R5;
+  case 6:
+    return S5;
+  case 7:
+    return T5;
+  case 8:
+    return W5;
+  default:
+    return B5;
+  }
+}
+
 void KasumiMainWindow::setActiveWordClass(WordClassType type){
   switch(type){
   case NOUN:
@@ -764,6 +985,9 @@ void KasumiMainWindow::setActiveWordClass(WordClassType type){
   case ADJ:
     gtk_combo_box_set_active(GTK_COMBO_BOX(WordClassCombo),4);
     break;
+  case VERB:
+    gtk_combo_box_set_active(GTK_COMBO_BOX(WordClassCombo),5);
+    break;
   default:
     gtk_combo_box_set_active(GTK_COMBO_BOX(WordClassCombo),0);
     break;
@@ -782,6 +1006,8 @@ WordClassType KasumiMainWindow::getActiveWordClass(){
     return PLACE;
   case 4:
     return ADJ;
+  case 5:
+    return VERB;
   default:
     return NOUN;
   }
@@ -898,6 +1124,15 @@ void KasumiMainWindow::synchronizeOptionCheckButton(KasumiWord *word){
                              HandlerIDOfAdvOptionSuruConnectionCheck);
     g_signal_handler_unblock(AdvOptionGokanCheck,
                              HandlerIDOfAdvOptionGokanCheck);
+  }else if(word->getWordClass() == VERB){
+    g_signal_handler_block(VerbOptionRentaiCheck,
+                           HandlerIDOfVerbOptionRentaiCheck);
+    gtk_toggle_button_set_active(
+      GTK_TOGGLE_BUTTON(VerbOptionRentaiCheck),
+      word->getOption(string(EUCJP_RENNYOUKEINOMEISHIKA)));
+    g_signal_handler_unblock(VerbOptionRentaiCheck,
+                             HandlerIDOfVerbOptionRentaiCheck);
+
   }
 }
 
@@ -905,32 +1140,34 @@ void KasumiMainWindow::flipOptionPane(){
   if(getActiveWordClass() == NOUN){
     gtk_widget_show(NounOptionPane);
     gtk_widget_hide(AdvOptionPane);
+    gtk_widget_hide(VerbOptionPane);
   }else if(getActiveWordClass() == ADV){
     gtk_widget_hide(NounOptionPane);
     gtk_widget_show(AdvOptionPane);
+    gtk_widget_hide(VerbOptionPane);
+  }else if(getActiveWordClass() == VERB){
+    gtk_widget_hide(NounOptionPane);
+    gtk_widget_hide(AdvOptionPane);
+    gtk_widget_show(VerbOptionPane);
   }else{
     gtk_widget_hide(NounOptionPane);
     gtk_widget_hide(AdvOptionPane);
+    gtk_widget_hide(VerbOptionPane);
+    
   }
 }
 
-void _call_back_destroy(GtkWidget *widget,
-                        gpointer data){
+void _call_back_delete_event(GtkWidget *widget,
+                             GdkEvent *event,
+                             gpointer data){
   KasumiMainWindow *window = (KasumiMainWindow *)data;
-  window->destroy();
-}
-
-gboolean _call_back_delete_event(GtkWidget *widget,
-                                 GdkEvent *event,
-                                 gpointer data){
-  KasumiMainWindow *window = (KasumiMainWindow *)data;
-  return window->delete_event(event);
+  window->quit();
 }
 
 void _call_back_quit(GtkWidget *widget,
                      gpointer data){
   KasumiMainWindow *window = (KasumiMainWindow *)data;
-  window->ClickedQuitButton();
+  window->quit();
 }
 
 void _call_back_store(GtkWidget *widget,
@@ -951,6 +1188,12 @@ void _call_back_remove(GtkWidget *widget,
   window->ClickedRemoveButton();
 }
 
+void _call_back_adding_mode(GtkWidget *widget,
+                            gpointer data){
+  KasumiMainWindow *window = (KasumiMainWindow *)data;
+  window->SwitchToAddingMode();
+}
+
 void _call_back_changed_list_cursor(GtkWidget *widget,
                                     gpointer data){
   KasumiMainWindow *window = (KasumiMainWindow *)data;
@@ -981,21 +1224,94 @@ void _call_back_changed_word_class_combo(GtkWidget *widget,
   window->ChangedWordClassCombo();
 }
 
+void _call_back_changed_verb_type_combo(GtkWidget *widget,
+                                         gpointer data){
+  KasumiMainWindow *window = (KasumiMainWindow *)data;
+  window->ChangedVerbTypeCombo();
+}
+
 void _call_back_toggled_check(GtkWidget *widget,
                               gpointer data){
   KasumiMainWindow *window = (KasumiMainWindow *)data;
   window->ChangedOption(widget);
 }
 
-void _call_back_find_next_by_sound(GtkWidget *widget,
-                                   gpointer data){
+void _call_back_changed_search_entry(GtkWidget *widget,
+                                       gpointer data){
   KasumiMainWindow *window = (KasumiMainWindow *)data;
-  window->FindNextBySound(widget);
+  window->FindNext(true);
 }
 
-void _call_back_find_next_by_spelling(GtkWidget *widget,
-                                      gpointer data){
+void _call_back_activate_search_entry(GtkWidget *widget,
+                                       gpointer data){
   KasumiMainWindow *window = (KasumiMainWindow *)data;
-  window->FindNextBySpelling(widget);
+  window->FindNext(false);
 }
 
+guint getAccelKey(const string &key){
+  string::size_type i;
+
+  i = key.find("+",0);
+
+  string shortkey = key.substr(i+1);
+
+  if(shortkey == "A") return GDK_A;
+  else if(shortkey == "B") return GDK_B;
+  else if(shortkey == "C") return GDK_C;
+  else if(shortkey == "D") return GDK_D;
+  else if(shortkey == "E") return GDK_E;
+  else if(shortkey == "F") return GDK_F;
+  else if(shortkey == "G") return GDK_G;
+  else if(shortkey == "H") return GDK_H;
+  else if(shortkey == "I") return GDK_I;
+  else if(shortkey == "J") return GDK_J;
+  else if(shortkey == "K") return GDK_K;
+  else if(shortkey == "L") return GDK_L;
+  else if(shortkey == "M") return GDK_M;
+  else if(shortkey == "N") return GDK_N;
+  else if(shortkey == "O") return GDK_O;
+  else if(shortkey == "P") return GDK_P;
+  else if(shortkey == "Q") return GDK_Q;
+  else if(shortkey == "R") return GDK_R;
+  else if(shortkey == "S") return GDK_S;
+  else if(shortkey == "T") return GDK_T;
+  else if(shortkey == "U") return GDK_U;
+  else if(shortkey == "V") return GDK_V;
+  else if(shortkey == "W") return GDK_W;
+  else if(shortkey == "X") return GDK_X;
+  else if(shortkey == "Y") return GDK_Y;
+  else if(shortkey == "Z") return GDK_Z;
+  else if(shortkey == "0") return GDK_0;
+  else if(shortkey == "1") return GDK_1;
+  else if(shortkey == "2") return GDK_2;
+  else if(shortkey == "3") return GDK_3;
+  else if(shortkey == "4") return GDK_4;
+  else if(shortkey == "5") return GDK_5;
+  else if(shortkey == "6") return GDK_5;
+  else if(shortkey == "7") return GDK_7;
+  else if(shortkey == "8") return GDK_8;
+  else if(shortkey == "9") return GDK_9;
+  
+  cout << "Invalid shortcut key option: " << key << endl;
+  exit(1);
+}
+
+GdkModifierType getModifierType(const string &key){
+  string::size_type i;
+
+  i = key.find("+",0);
+
+  if(i == key.npos){
+    return (GdkModifierType)0;
+  }
+
+  string mask = key.substr(0,i);
+  if(mask == "Ctrl"){
+    return GDK_CONTROL_MASK;
+  }else if(mask == "Alt"){
+    return GDK_MOD1_MASK;
+  }
+
+  cout << "Invalid mask option: " << key << endl;
+  exit(1);
+}
