@@ -1,5 +1,6 @@
 #include "KasumiString.hxx"
 #include <iostream>
+#include <sstream>
 
 using namespace std;
 
@@ -21,7 +22,7 @@ bool KasumiString::isEmptyLine(){
 }
 
 bool KasumiString::isEntryLine(){
-  unsigned int i;
+  string::size_type i;
 
   if(isKeyValLine()){
     return false;
@@ -37,16 +38,11 @@ bool KasumiString::isEntryLine(){
     return false;
   }
 
-  i = find(" ",i+1);
-  if(i == npos){
-    return true;
-  }
-
-  return false;
+  return true;
 }
 
 bool KasumiString::isKeyValLine(){
-  unsigned int i;
+  string::size_type i;
 
   i = find("=",0);
 
@@ -58,7 +54,7 @@ bool KasumiString::isKeyValLine(){
 }
 
 string KasumiString::getSound(){
-  unsigned int i;
+  string::size_type i;
 
   if(isEntryLine()){
     i = find(" ",0);
@@ -69,7 +65,8 @@ string KasumiString::getSound(){
 }
 
 unsigned int KasumiString::getFrequency(){
-  unsigned int i,j,ret = 0;
+  string::size_type i,j;
+  unsigned int ret=0;
   string sub;
 
   if(isEntryLine()){
@@ -88,7 +85,10 @@ string KasumiString::getSpelling(){
   int i;
   
   if(isEntryLine()){
-    i = rfind(" ",npos);
+    // allow spelling to contain space
+    // thanks to pinkmonky http://d.hatena.ne.jp/pinkmonkey/20050411
+    i = find(" ",0);
+    i = find(" ",i+1);
     return substr(i+1);
   }
 
@@ -124,3 +124,50 @@ string KasumiString::getVal(){
 
   return NULL;
 }
+
+int str2int(const string &str){
+  if(isInt(str)){
+    stringstream ss(str);
+    int i;
+    ss >> i;
+    return i;
+  }
+  
+  return 0;
+}
+
+string int2str(int i){
+   stringstream ss;
+   ss << i;
+   return ss.str();
+}
+
+bool isInt(const string &str){
+  unsigned int i;
+  
+  if(str == "-" || str == ""){
+    return false;
+  }
+
+  if(str == "0"){
+    return true;
+  }
+
+  if(str.c_str()[0] == '-'){
+    i=1;
+  }else{
+    i=0;
+  }
+
+  if(str.c_str()[i] >= '1' && str.c_str()[i] <= '9'){
+    for(i++;i<str.length();i++){
+      if(str.c_str()[i] < '0' || str.c_str()[i] > '9'){
+        return false;
+      }
+    }
+
+    return true;
+  }
+
+  return false;
+}
